# -*- coding: utf-8 -*-
"""
User-interface related functions for building bots.

Note: the script requires the Python IRC library
http://python-irclib.sourceforge.net/
"""
#
# (C) Balasyum, 2008
# (C) Pywikibot team, 2008-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

# Note: the intention is to develop this module (at some point) into a Bot
# class definition that can be subclassed to create new, functional bot
# scripts, instead of writing each one from scratch.


import re

import pywikibot

try:
    from ircbot import SingleServerIRCBot
except ImportError as e:
    ircbot_import_error = e

    class SingleServerIRCBot(object):

        """Fake SingleServerIRCBot."""

        def __init__(*args, **kwargs):
            """Report import exception."""
            raise ircbot_import_error


_logger = 'botirc'


class IRCBot(pywikibot.Bot, SingleServerIRCBot):

    """
    A generic IRC Bot to be subclassed.

    A Bot that displays the ordinal number of the new articles being created
    visible on the Recent Changes list. The Bot doesn't make any edits, no
    account needed.
    """

    # Bot configuration.
    # Only the keys of the dict can be passed as init options
    # The values are the default values
    # Extend this in subclasses!
    availableOptions = {}  # noqa: N815

    def __init__(self, site, channel, nickname, server, port=6667, **kwargs):
        """Initializer."""
        pywikibot.Bot.__init__(self, **kwargs)
        SingleServerIRCBot.__init__(self, [(server, port)], nickname, nickname)
        self.channel = channel
        self.site = site
        self.other_ns = re.compile(
            '\x0314\\[\\[\x0307(%s)'
            % '|'.join(item.custom_name for item in site.namespaces.values()
                       if item != 0))
        self.api_url = (
            self.site.apipath()
            + '?action=query&meta=siteinfo&siprop=statistics&format=xml')
        self.api_found = re.compile(r'articles="(.*?)"')
        self.re_edit = re.compile(
            r'^C14\[\[^C07(?P<page>.+?)^C14\]\]^C4 (?P<flags>.*?)^C10 ^C02'
            r'(?P<url>.+?)^C ^C5\*^C ^C03(?P<user>.+?)^C ^C5\*^C \(?^B?'
            r'(?P<bytes>[+-]?\d+?)^B?\) ^C10(?P<summary>.*)^C'
            .replace('^B', '\002').replace('^C', '\003').replace('^U', '\037'))

    def on_nicknameinuse(self, c, e):
        """Provide an alternative nickname."""
        c.nick(c.get_nickname() + '_')

    def on_welcome(self, c, e):
        """Join channel."""
        c.join(self.channel)

    def on_privmsg(self, c, e):
        """Ignore private message."""
        pass

    def on_pubmsg(self, c, e):
        """Respond to public message."""
        match = self.re_edit.match(e.arguments()[0])
        if not match:
            return
        if not ('N' in match.group('flags')):
            return
        try:
            msg = e.arguments()[0].decode('utf-8')
        except UnicodeDecodeError:
            return
        if self.other_ns.match(msg):
            return
        name = msg[8:msg.find('14', 9)]
        text = pywikibot.comms.http.request(self.site, self.api_url)
        entry = self.api_found.findall(text)
        page = pywikibot.Page(self.site, name)
        try:
            text = page.get()
        except pywikibot.NoPage:
            return
        except pywikibot.IsRedirectPage:
            return
        pywikibot.output(str((entry[0], name)))

    def on_dccmsg(self, c, e):
        """Ignore DCC message."""
        pass

    def on_dccchat(self, c, e):
        """Ignore DCC chat."""
        pass

    def do_command(self, e, cmd):
        """Ignore command request."""
        pass

    def on_quit(self, e, cmd):
        """Ignore quit request."""
        pass



# -*- coding: utf-8 -*-
"""IP address tools module."""
#
# (C) Pywikibot team, 2014-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

import re

from distutils.version import StrictVersion
from warnings import warn

from pywikibot.tools import DeprecatedRegex, PY2, UnicodeType

_ipaddress_e = _ipaddr_e = _ipaddr_version = None

try:
    from ipaddress import ip_address
except ImportError as e:
    _ipaddress_e = e
    ip_address = None

if not ip_address or PY2:
    try:
        from ipaddr import __version__ as _ipaddr_version
    except ImportError as e:
        _ipaddr_e = e
    else:
        _ipaddr_version = StrictVersion(_ipaddr_version)
        if _ipaddr_version >= StrictVersion('2.1.10'):
            from ipaddr import IPAddress as ip_address  # noqa: N813
        else:
            _ipaddr_e = ImportError('ipaddr %s is broken.' % _ipaddr_version)

if ip_address and ip_address.__module__ == 'ipaddress':
    if PY2:
        # This backport fails many tests
        # https://pypi.org/project/py2-ipaddress
        # It accepts u'1111' as a valid IP address.
        try:
            ip_address('1111')
            ip_address = None
            raise ImportError('ipaddress backport is broken; install ipaddr')
        except ValueError:
            pass

        # This backport only fails a few tests if given a unicode object
        # https://pypi.org/project/ipaddress
        # However while it rejects u'1111', it will consider '1111' valid
        try:
            ip_address(b'1111')
            warn('ipaddress backport is defective; patching; install ipaddr',
                 ImportWarning)
            orig_ip_address = ip_address
            # force all input to be a unicode object so it validates correctly

            def ip_address_patched(IP):
                """Safe ip_address."""
                return orig_ip_address(UnicodeType(IP))

            ip_address = ip_address_patched
        except ValueError:
            # This means ipaddress has correctly determined '1111' is invalid
            pass
elif not ip_address:
    warn('Importing ipaddr.IPAddress failed: %s\n'
         'Importing ipaddress.ip_address failed: %s\n'
         'Please install ipaddr 2.1.10+ or ipaddress.'
         % (_ipaddr_e, _ipaddress_e), ImportWarning)

    def ip_address_fake(IP):
        """Fake ip_address method."""
        warn('ipaddress backport not available.', DeprecationWarning)
        if ip_regexp.match(IP) is None:
            raise ValueError('Invalid IP address')

    ip_address = ip_address_fake

# deprecated IP detector
ip_regexp = DeprecatedRegex(
    r'^(?:(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}'
    r'(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)|'
    r'(((?=(?=(.*?(::)))\3(?!.+\4)))\4?|[\dA-F]{1,4}:)'
    r'([\dA-F]{1,4}(\4|:\b)|\2){5}'
    r'(([\dA-F]{1,4}(\4|:\b|$)|\2){2}|'
    r'(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4}))\Z',
    re.IGNORECASE,
    'page.ip_regexp', 'tools.ip.is_IP', since='20150212')


def is_IP(IP):
    """
    Verify the IP address provided is valid.

    No logging is performed. Use ip_address instead to catch errors.

    @param IP: IP address
    @type IP: str
    @rtype: bool
    """
    try:
        ip_address(IP)
        return True
    except ValueError:
        return False



# -*- coding: utf-8 -*-
"""Family module for WOW Wiki."""
#
# (C) Pywikibot team, 2009-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family
from pywikibot.tools import deprecated, classproperty


class Family(family.SubdomainFamily, family.FandomFamily):

    """Family class for WOW Wiki."""

    name = 'wowwiki'
    domain = 'wowwiki.fandom.com'

    codes = (
        'ar', 'cs', 'da', 'de', 'el', 'en', 'es', 'et', 'fa', 'fi', 'fr', 'he',
        'hu', 'is', 'it', 'ja', 'ko', 'lt', 'lv', 'nl', 'nn', 'no', 'pl', 'pt',
        'pt-br', 'ru', 'sk', 'tr', 'uk', 'zh', 'zh-tw'
    )

    interwiki_removals = ['hr', 'ro', 'sr', 'sv']

    @classproperty
    @deprecated('codes attribute', since='20190422')
    def languages_by_size(cls):
        """DEPRECATED. languages_by_size property for compatibility purpose."""
        return list(cls.codes)

    @classproperty
    def langs(cls):
        """Property listing family languages."""
        cls.langs = {code: cls.domain for code in cls.codes}
        cls.langs.update({code: cls.domains[1] for code in ('es', 'et')})
        cls.langs['uk'] = 'uk.' + cls.domains[2]
        return cls.langs

    @classproperty
    def disambiguationTemplates(cls):
        """Property listing disambiguation templates."""
        cls.disambiguationTemplates = \
            super(Family, cls).disambiguationTemplates
        cls.disambiguationTemplates['en'] = ['disambig', 'disambig/quest',
                                             'disambig/quest2',
                                             'disambig/achievement2']
        return cls.disambiguationTemplates

    @classproperty
    def disambcatname(cls):
        """Property listing disambiguation category name."""
        cls.disambcatname = super(Family, cls).disambcatname
        cls.disambcatname['en'] = 'Disambiguations'
        return cls.disambcatname

    # Wikia's default CategorySelect extension always puts categories last
    @classproperty
    def categories_last(cls):
        """Property listing site keys for categories at last position."""
        return cls.langs.keys()

    @classproperty
    def domains(cls):
        """List of domains used by family wowwiki."""
        return [cls.domain, 'worldofwarcraft.fandom.com', 'warcraft.wikia.com']

    @deprecated('APISite.version()', since='20141225')
    def version(self, code):
        """Return the version for this family."""
        return '1.19.24'

    def protocol(self, code):
        """Return the protocol for this family."""
        return 'http' if code == 'uk' else 'https'

    def scriptpath(self, code):
        """Return the script path for this family."""
        if code == 'uk':
            return ''
        return super(Family, self).scriptpath(code)



# -*- coding: utf-8 -*-
"""Family module for Wikibooks."""
#
# (C) Pywikibot team, 2005-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family


# The Wikimedia family that is known as Wikibooks
class Family(family.SubdomainFamily, family.WikimediaFamily):

    """Family class for Wikibooks."""

    name = 'wikibooks'

    closed_wikis = [
        # See https://noc.wikimedia.org/conf/highlight.php?file=closed.dblist
        'aa', 'ak', 'als', 'ang', 'as', 'ast', 'ay',
        'bi', 'bm', 'bo', 'ch', 'co', 'ga', 'gn', 'got',
        'gu', 'ie', 'kn', 'ks', 'lb', 'ln', 'lv', 'mi',
        'mn', 'my', 'na', 'nah', 'nds', 'ps', 'qu', 'rm',
        'se', 'simple', 'su', 'sw', 'tk', 'ug', 'uz',
        'vo', 'wa', 'xh', 'yo', 'za', 'zh-min-nan', 'zu',
    ]

    removed_wikis = [
        # See https://noc.wikimedia.org/conf/highlight.php?file=closed.dblist
        'dk', 'tokipona',
    ]

    languages_by_size = [
        'en', 'hu', 'de', 'fr', 'pt', 'ja', 'it', 'es', 'nl', 'pl', 'he', 'th',
        'id', 'fi', 'sq', 'az', 'fa', 'zh', 'ca', 'ru', 'vi', 'da', 'ko', 'sv',
        'gl', 'sr', 'cs', 'hr', 'ba', 'no', 'ar', 'tr', 'ta', 'sa', 'uk', 'eo',
        'sk', 'is', 'ro', 'hi', 'si', 'bn', 'mk', 'bg', 'ka', 'ms', 'lt', 'tt',
        'el', 'li', 'ur', 'sl', 'km', 'tl', 'et', 'kk', 'ia', 'be', 'ml', 'oc',
        'hy', 'eu', 'ne', 'tg', 'la', 'cv', 'pa', 'fy', 'ku', 'bs', 'cy', 'af',
        'te', 'mr', 'mg', 'ky',
    ]

    category_redirect_templates = {
        '_default': (),
        'en': ('Category redirect',),
        'es': ('Categoría redirigida',),
        'ro': ('Redirect categorie',),
        'vi': ('Đổi hướng thể loại',),
    }

    # Global bot allowed languages on
    # https://meta.wikimedia.org/wiki/BPI#Current_implementation
    cross_allowed = [
        'af', 'ca', 'fa', 'fy', 'gl', 'it', 'nl', 'ru', 'th', 'zh',
    ]

    # Subpages for documentation.
    # TODO: List is incomplete, to be completed for missing languages.
    doc_subpages = {
        '_default': (('/doc', ),
                     ['en']
                     ),
        'es': ('/uso', '/doc'),
    }



# -*- coding: utf-8 -*-
"""Family module for Wikimedia Strategy Wiki."""
#
# (C) Pywikibot team, 2009-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family


# The Wikimedia Strategy family
class Family(family.WikimediaOrgFamily):

    """Family class for Wikimedia Strategy Wiki."""

    name = 'strategy'

    interwiki_forward = 'wikipedia'

    def dbName(self, code):
        """Return the database name for this family."""
        return 'strategywiki_p'



# -*- coding: utf-8 -*-
"""
WARNING: THIS MODULE EXISTS SOLELY TO PROVIDE BACKWARDS-COMPATIBILITY.

Do not use in new scripts; use the source to find the appropriate
function/method instead.

"""
#
# (C) Pywikibot team, 2008-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import Category
from pywikibot.tools import ModuleDeprecationWrapper


def change_category(article, oldCat, newCat, comment=None, sortKey=None,
                    inPlace=True):
    """Change the category of the article."""
    return article.change_category(oldCat, newCat, comment, sortKey, inPlace)


__all__ = ('Category', 'change_category',)

wrapper = ModuleDeprecationWrapper(__name__)
wrapper._add_deprecated_attr('Category',
                             replacement_name='pywikibot.Category',
                             since='20141209')
wrapper._add_deprecated_attr('change_category',
                             replacement_name='Page.change_category',
                             since='20141209')



# -*- coding: utf-8 -*-
"""CGI user interface."""
#
# (C) Pywikibot team, 2007-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

import sys


class UI(object):

    """CGI user interface."""

    def output(self, text, colors=None, newline=True, toStdout=False):
        """Output text to CGI stream if toStdout is True."""
        if not toStdout:
            return
        sys.stdout.write(text.encode('UTF-8', 'replace'))

    def input(self, question, colors=None):
        """Output question to CGI stream."""
        self.output(question + ' ', newline=False, toStdout=True)



# -*- coding: utf-8 -*-
"""Module containing plural rules of various languages."""
#
# (C) xqt, 2011-2017
# (C) Pywikibot team, 2011-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals


plural_rules = {
    '_default': {'nplurals': 2, 'plural': lambda n: (n != 1)},
    'ach': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'ak': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'am': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'ar': {'nplurals': 6, 'plural': lambda n:
           0 if (n == 0) else
           1 if (n == 1) else
           2 if (n == 2) else
           3 if (n % 100 >= 3 and n % 100 <= 10) else
           4 if (n % 100 >= 11 and n % 100 <= 99) else
           5},
    'arn': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'ay': {'nplurals': 1, 'plural': 0},
    'be': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'bo': {'nplurals': 1, 'plural': 0},
    'br': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'bs': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'cgg': {'nplurals': 1, 'plural': 0},
    'cs': {'nplurals': 3, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n >= 2 and n <= 4) else
           2},
    'csb': {'nplurals': 3, 'plural': lambda n:
            0 if (n == 1) else
            1 if n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20) else
            2},
    'cy': {'nplurals': 4, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n == 2) else
           2 if (n != 8 and n != 11) else
           3},
    'dz': {'nplurals': 1, 'plural': 0},
    'fa': {'nplurals': 1, 'plural': 0},
    'fil': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'fr': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'ga': {'nplurals': 5, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n == 2) else
           2 if (n < 7) else
           3 if (n < 11) else
           4},
    'gd': {'nplurals': 4, 'plural': lambda n:
           0 if (n == 1 or n == 11) else
           1 if (n == 2 or n == 12) else
           2 if (n > 2 and n < 20) else
           3},
    'gun': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'hr': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'id': {'nplurals': 1, 'plural': 0},
    'is': {'nplurals': 2, 'plural': lambda n: (n % 10 != 1 or n % 100 == 11)},
    'ja': {'nplurals': 1, 'plural': 0},
    'jbo': {'nplurals': 1, 'plural': 0},
    'ka': {'nplurals': 1, 'plural': 0},
    'kk': {'nplurals': 1, 'plural': 0},
    'km': {'nplurals': 1, 'plural': 0},
    'ko': {'nplurals': 1, 'plural': 0},
    'kw': {'nplurals': 4, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n == 2) else
           2 if (n == 3) else
           3},
    'ky': {'nplurals': 1, 'plural': 0},
    'ln': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'lo': {'nplurals': 1, 'plural': 0},
    'lt': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 >= 2 and (n % 100 < 10 or n % 100 >= 20)) else
           2},
    'lv': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n != 0) else
           2},
    'mfe': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'mg': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'mi': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'mk': {'nplurals': 2, 'plural': lambda n:
           0 if n == 1 or n % 10 == 1 else
           1},
    'mnk': {'nplurals': 3, 'plural': lambda n:
            0 if (n == 0) else
            1 if n == 1 else
            2},
    'ms': {'nplurals': 1, 'plural': 0},
    'mt': {'nplurals': 4, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n == 0 or (n % 100 > 1 and n % 100 < 11)) else
           2 if (n % 100 > 10 and n % 100 < 20) else
           3},
    'my': {'nplurals': 1, 'plural': 0},
    'oc': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'pl': {'nplurals': 3, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'pt-br': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'ro': {'nplurals': 3, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n == 0 or (n % 100 > 0 and n % 100 < 20)) else
           2},
    'ru': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'sah': {'nplurals': 1, 'plural': 0},
    'sk': {'nplurals': 3, 'plural': lambda n:
           0 if (n == 1) else
           1 if (n >= 2 and n <= 4) else
           2},
    'sl': {'nplurals': 4, 'plural': lambda n:
           0 if (n % 100 == 1) else
           1 if (n % 100 == 2) else
           2 if (n % 100 == 3 or n % 100 == 4) else
           3},
    'sr': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'su': {'nplurals': 1, 'plural': 0},
    'th': {'nplurals': 1, 'plural': 0},
    'ti': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'tr': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'tt': {'nplurals': 1, 'plural': 0},
    'ug': {'nplurals': 1, 'plural': 0},
    'uk': {'nplurals': 3, 'plural': lambda n:
           0 if (n % 10 == 1 and n % 100 != 11) else
           1 if (n % 10 in range(2, 5) and (n % 100 < 10 or n % 100 >= 20))
           else 2},
    'uz': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'vi': {'nplurals': 1, 'plural': 0},
    'wa': {'nplurals': 2, 'plural': lambda n: (n > 1)},
    'wo': {'nplurals': 1, 'plural': 0},
    'zh': {'nplurals': 1, 'plural': 0},
    'zh-hans': {'nplurals': 1, 'plural': 0},
    'zh-hant': {'nplurals': 1, 'plural': 0},
    'zh-tw': {'nplurals': 1, 'plural': 0},
}



# -*- coding: utf-8 -*-
"""Functions for manipulating external links or querying third-party sites."""
#
# (C) Pywikibot team, 2013-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

import json
from time import sleep
from xml.etree import ElementTree

from requests.exceptions import ConnectionError as RequestsConnectionError

from pywikibot.comms import http
from pywikibot import config2
from pywikibot.tools import deprecated, PY2

if not PY2:
    from urllib.parse import urlencode
else:
    from urllib import urlencode


@deprecated('memento_client package', since='20150811')
def getInternetArchiveURL(url, timestamp=None):
    """Return archived URL by Internet Archive.

    See [[:mw:Archived Pages]] and https://archive.org/help/wayback_api.php
    for more details.

    @param url: url to search an archived version for
    @param timestamp: requested archive date. The version closest to that
        moment is returned. Format: YYYYMMDDhhmmss or part thereof.

    """
    uri = 'https://archive.org/wayback/available?'

    query = {'url': url}

    if timestamp is not None:
        query['timestamp'] = timestamp

    uri = uri + urlencode(query)

    retry_count = 0
    while retry_count <= config2.max_retries:
        try:
            jsontext = http.fetch(uri).text
            break
        except RequestsConnectionError as e:
            error = e
            retry_count += 1
            sleep(config2.retry_wait)
    else:
        raise error

    if 'closest' in jsontext:
        data = json.loads(jsontext)
        return data['archived_snapshots']['closest']['url']
    else:
        return None


@deprecated('memento_client package', since='20150811')
def getWebCitationURL(url, timestamp=None):
    """Return archived URL by Web Citation.

    See http://www.webcitation.org/doc/WebCiteBestPracticesGuide.pdf
    for more details

    @param url: url to search an archived version for
    @param timestamp: requested archive date. The version closest to that
        moment is returned. Format: YYYYMMDDhhmmss or part thereof.

    """
    uri = 'http://www.webcitation.org/query?'

    query = {'returnxml': 'true',
             'url': url}

    if timestamp is not None:
        query['date'] = timestamp

    uri = uri + urlencode(query)
    xmltext = http.fetch(uri).text
    if 'success' in xmltext:
        data = ElementTree.fromstring(xmltext)
        return data.find('.//webcite_url').text
    else:
        return None



# -*- coding: utf-8 -*-
"""Family module for Incubator Wiki."""
#
# (C) Pywikibot team, 2006-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family


# The Wikimedia Incubator family
class Family(family.WikimediaOrgFamily):

    """Family class for Incubator Wiki."""

    name = 'incubator'



# -*- coding: utf-8 -*-
"""Family module for test.wikipedia.org."""
#
# (C) Pywikibot team, 2007-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family


# The test wikipedia family
class Family(family.SingleSiteFamily, family.WikimediaFamily):

    """Family class for test.wikipedia.org."""

    name = 'test'
    domain = 'test.wikipedia.org'
    test_codes = ('test', )



# -*- coding: utf-8 -*-
"""Family module for Wikipedia."""
#
# (C) Pywikibot team, 2004-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family


# The Wikimedia family that is known as Wikipedia, the Free Encyclopedia
class Family(family.SubdomainFamily, family.WikimediaFamily):

    """Family module for Wikipedia."""

    name = 'wikipedia'

    closed_wikis = [
        # See https://noc.wikimedia.org/conf/highlight.php?file=closed.dblist
        'aa', 'advisory', 'cho', 'ho', 'hz', 'ii', 'kj', 'kr', 'mh', 'mo',
        'mus', 'ng', 'quality', 'strategy', 'ten', 'usability'
    ]

    removed_wikis = [
        # See https://noc.wikimedia.org/conf/highlight.php?file=deleted.dblist
        'dk', 'ru-sib', 'tlh', 'tokipona', 'zh_cn', 'zh_tw',
    ]

    languages_by_size = [
        'en', 'ceb', 'sv', 'de', 'fr', 'nl', 'ru', 'it', 'es', 'pl', 'war',
        'vi', 'ja', 'zh', 'pt', 'uk', 'ar', 'fa', 'sr', 'ca', 'no', 'id', 'fi',
        'ko', 'hu', 'sh', 'cs', 'ro', 'eu', 'tr', 'ms', 'eo', 'bg', 'da', 'hy',
        'he', 'sk', 'zh-min-nan', 'kk', 'min', 'ce', 'hr', 'lt', 'et', 'be',
        'sl', 'el', 'gl', 'az', 'ur', 'simple', 'nn', 'azb', 'uz', 'th', 'hi',
        'la', 'ka', 'vo', 'ta', 'cy', 'mk', 'ast', 'tg', 'lv', 'mg', 'oc',
        'tt', 'bs', 'ky', 'af', 'sq', 'tl', 'zh-yue', 'new', 'te', 'be-tarask',
        'br', 'bn', 'pms', 'ml', 'lb', 'jv', 'ht', 'sco', 'mr', 'ga', 'sw',
        'ba', 'pnb', 'nds', 'is', 'my', 'fy', 'cv', 'su', 'lmo', 'an', 'ne',
        'yo', 'pa', 'gu', 'io', 'bar', 'scn', 'ku', 'als', 'bpy', 'kn', 'ckb',
        'ia', 'qu', 'arz', 'wuu', 'mn', 'bat-smg', 'si', 'wa', 'or', 'gd',
        'yi', 'am', 'nap', 'cdo', 'bug', 'hsb', 'map-bms', 'mai', 'xmf', 'fo',
        'mzn', 'li', 'ilo', 'sah', 'eml', 'vec', 'os', 'sa', 'sd', 'mrj',
        'mhr', 'diq', 'ps', 'hif', 'roa-tara', 'zh-classical', 'bcl', 'ace',
        'hak', 'frr', 'pam', 'szl', 'nso', 'se', 'km', 'nv', 'mi', 'rue',
        'nah', 'nds-nl', 'vls', 'bh', 'hyw', 'gan', 'crh', 'sc', 'vep', 'bo',
        'glk', 'co', 'myv', 'tk', 'fiu-vro', 'as', 'so', 'kv', 'lrc', 'csb',
        'gv', 'shn', 'zea', 'udm', 'ie', 'pcd', 'ay', 'sn', 'ug', 'nrm', 'stq',
        'lez', 'lad', 'kw', 'mwl', 'gom', 'gn', 'haw', 'rm', 'lij', 'ha',
        'koi', 'mt', 'lfn', 'fur', 'frp', 'lo', 'dsb', 'kab', 'ab', 'ln',
        'ang', 'ext', 'olo', 'dty', 'cbk-zam', 'dv', 'ksh', 'gag', 'pi', 'pfl',
        'pag', 'av', 'bjn', 'bxr', 'xal', 'gor', 'krc', 'pap', 'za', 'kaa',
        'pdc', 'rw', 'tyv', 'to', 'kl', 'nov', 'jam', 'arc', 'kbp', 'kbd',
        'tpi', 'tet', 'ig', 'ki', 'na', 'jbo', 'lbe', 'roa-rup', 'ty', 'bi',
        'kg', 'mdf', 'wo', 'lg', 'zu', 'srn', 'tcy', 'atj', 'inh', 'chr',
        'ltg', 'sm', 'xh', 'om', 'pih', 'sat', 'cu', 'rmy', 'tw', 'bm', 'tn',
        'chy', 'rn', 'got', 'ts', 'tum', 'ak', 'st', 'ny', 'ch', 'ss', 'pnt',
        'fj', 'iu', 'ady', 'ee', 'ks', 've', 'ik', 'sg', 'ff', 'dz', 'ti',
        'cr', 'din',
    ]

    # Sites we want to edit but not count as real languages
    test_codes = ['test', 'test2']

    # Templates that indicate a category redirect
    # Redirects to these templates are automatically included
    category_redirect_templates = {
        '_default': (),
        'ar': ('تحويل تصنيف',),
        'arz': ('تحويل تصنيف',),
        'bn': ('বিষয়শ্রেণী পুনর্নির্দেশ',),
        'bs': ('Category redirect',),
        'cs': ('Zastaralá kategorie',),
        'da': ('Kategoriomdirigering',),
        'en': ('Category redirect',),
        'es': ('Categoría redirigida',),
        'eu': ('Kategoria birzuzendu',),
        'fa': ('رده بهتر',),
        'fr': ('Catégorie redirigée',),
        'gv': ('Aastiurey ronney',),
        'hi': ('श्रेणी अनुप्रेषित',),
        'hu': ('Kat-redir',),
        'id': ('Alih kategori',),
        'ja': ('Category redirect',),
        'ko': ('분류 넘겨주기',),
        'mk': ('Премести категорија',),
        'ml': ('Category redirect',),
        'ms': ('Pengalihan kategori',),
        'mt': ('Rindirizzament kategorija',),
        'ne': ('श्रेणी अनुप्रेषण',),
        'no': ('Kategoriomdirigering',),
        'pt': ('Redirecionamento de categoria',),
        'ro': ('Redirect categorie',),
        'ru': ('Переименованная категория',),
        'sco': ('Category redirect',),
        'sh': ('Prekat',),
        'simple': ('Category redirect',),
        'sl': ('Preusmeritev kategorije',),
        'sr': ('Category redirect',),
        'sq': ('Kategori e zhvendosur',),
        'sv': ('Kategoriomdirigering',),
        'tl': ('Category redirect',),
        'tr': ('Kategori yönlendirme',),
        'uk': ('Categoryredirect',),
        'ur': ('زمرہ رجوع مکرر',),
        'vi': ('Đổi hướng thể loại',),
        'yi': ('קאטעגאריע אריבערפירן',),
        'zh': ('分类重定向',),
        'zh-yue': ('分類彈去',),
    }

    # families that redirect their interlanguage links here.
    interwiki_forwarded_from = [
        'commons',
        'incubator',
        'meta',
        'species',
        'strategy',
        'test',
        'wikimania'
    ]

    # Global bot allowed languages on
    # https://meta.wikimedia.org/wiki/BPI#Current_implementation
    # & https://meta.wikimedia.org/wiki/Special:WikiSets/2
    cross_allowed = [
        'ab', 'ace', 'ady', 'af', 'ak', 'als', 'am', 'an', 'ang', 'ar',
        'arc', 'arz', 'as', 'ast', 'av', 'ay', 'az', 'ba', 'bar',
        'bat-smg', 'bcl', 'be', 'be-tarask', 'bg', 'bh', 'bi', 'bjn', 'bm',
        'bo', 'bpy', 'bug', 'bxr', 'ca', 'cbk-zam', 'cdo', 'ce', 'ceb',
        'ch', 'chr', 'chy', 'ckb', 'co', 'cr', 'crh', 'cs', 'csb', 'cu',
        'cv', 'cy', 'da', 'diq', 'dsb', 'dz', 'ee', 'el', 'eml', 'en',
        'eo', 'et', 'eu', 'ext', 'fa', 'ff', 'fi', 'fj', 'fo', 'frp',
        'frr', 'fur', 'ga', 'gag', 'gan', 'gd', 'glk', 'gn', 'gor', 'got',
        'gu', 'gv', 'ha', 'hak', 'haw', 'he', 'hi', 'hif', 'hr', 'hsb', 'ht',
        'hu', 'hy', 'ia', 'ie', 'ig', 'ik', 'ilo', 'io', 'iu', 'ja', 'jam',
        'jbo', 'jv', 'ka', 'kaa', 'kab', 'kdb', 'kg', 'ki', 'kk', 'kl',
        'km', 'kn', 'ko', 'koi', 'krc', 'ks', 'ku', 'kv', 'kw', 'ky', 'la',
        'lad', 'lb', 'lbe', 'lez', 'lfn', 'lg', 'li', 'lij', 'lmo', 'ln', 'lo',
        'lt', 'ltg', 'lv', 'map-bms', 'mdf', 'mg', 'mhr', 'mi', 'mk', 'ml',
        'mn', 'mrj', 'ms', 'mwl', 'my', 'myv', 'mzn', 'na', 'nah', 'nap',
        'nds-nl', 'ne', 'new', 'nl', 'no', 'nov', 'nrm', 'nso', 'nv', 'ny',
        'oc', 'olo', 'om', 'or', 'os', 'pa', 'pag', 'pam', 'pap', 'pdc',
        'pfl', 'pi', 'pih', 'pms', 'pnb', 'pnt', 'ps', 'qu', 'rm', 'rmy',
        'rn', 'roa-rup', 'roa-tara', 'ru', 'rue', 'rw', 'sa', 'sah', 'sc',
        'scn', 'sco', 'sd', 'se', 'sg', 'sh', 'si', 'simple', 'sk', 'sm',
        'sn', 'so', 'srn', 'ss', 'st', 'stq', 'su', 'sv', 'sw', 'szl',
        'ta', 'tcy', 'te', 'tet', 'tg', 'th', 'ti', 'tk', 'tl', 'tn', 'to',
        'tpi', 'tr', 'ts', 'tt', 'tum', 'tw', 'ty', 'tyv', 'udm', 'ug',
        'uz', 've', 'vec', 'vep', 'vls', 'vo', 'wa', 'war', 'wo', 'wuu',
        'xal', 'xh', 'xmf', 'yi', 'yo', 'za', 'zea', 'zh', 'zh-classical',
        'zh-min-nan', 'zh-yue', 'zu',
    ]

    # On most Wikipedias page names must start with a capital letter,
    # but some languages don't use this.
    nocapitalize = ['jbo']

    # Languages that used to be coded in iso-8859-1
    latin1old = [
        'de', 'en', 'et', 'es', 'ia', 'la', 'af', 'cs', 'fr', 'pt', 'sl',
        'bs', 'fy', 'vi', 'lt', 'fi', 'it', 'no', 'simple', 'gl', 'eu',
        'nds', 'co', 'mi', 'mr', 'id', 'lv', 'sw', 'tt', 'uk', 'vo', 'ga',
        'na', 'es', 'nl', 'da', 'dk', 'sv', 'test']

    # Subpages for documentation.
    # TODO: List is incomplete, to be completed for missing languages.
    # TODO: Remove comments for appropriate pages
    doc_subpages = {
        '_default': (('/doc', ),
                     ['ar', 'bn', 'cs', 'da', 'en', 'es', 'hr',
                      'hu', 'id', 'ilo', 'ja', 'ms',
                      'pt', 'ro', 'ru', 'simple', 'sh', 'vi', 'zh']
                     ),
        'bs': ('/dok', ),
        'ca': ('/ús', ),
        'de': ('Doku', '/Meta'),
        'dsb': ('/Dokumentacija', ),
        'eu': ('txantiloi dokumentazioa', '/dok'),
        'fa': ('/doc', '/توضیحات'),
        # fi: no idea how to handle this type of subpage at :Metasivu:
        'fi': ((), ),
        'fr': ('/Documentation',),
        'hsb': ('/Dokumentacija', ),
        'it': ('/Man', ),
        'ka': ('/ინფო', ),
        'ko': ('/설명문서', ),
        'no': ('/dok', ),
        'nn': ('/dok', ),
        'pl': ('/opis', ),
        'sk': ('/Dokumentácia', ),
        'sr': ('/док', ),
        'sv': ('/dok', ),
        'uk': ('/Документація', ),
        'ur': ('/doc', '/دستاویز'),
    }

    # Templates that indicate an edit should be avoided
    edit_restricted_templates = {
        'ar': ('تحرر',),
        'bs': ('Izmjena u toku',),
        'cs': ('Pracuje se',),
        'de': ('Inuse', 'In use', 'In bearbeitung', 'Inbearbeitung',),
        'en': ('Inuse', 'In use'),
        'fa': ('ویرایش',),
        'fr': ('En cours', 'Plusieurs en cours', 'Correction en cours',
               'Inuse', 'Remix',),
        'he': ('בעבודה',),
        'hr': ('Radovi',),
        'sr': ('Радови у току', 'Рут',),
        'ur': ('زیر ترمیم',),
        'zh': ('Inuse',),
    }

    # Archive templates that indicate an edit of non-archive bots
    # should be avoided
    archived_page_templates = {
        'cs': ('Archiv', 'Archiv Wikipedie', 'Archiv diskuse',
               'Archivace start', 'Posloupnost archivů',
               'Rfa-archiv-start', 'Rfc-archiv-start',),
        'de': ('Archiv',),
    }

    def get_known_families(self, site):
        """Override the family interwiki prefixes for each site."""
        # In Swedish Wikipedia 's:' is part of page title not a family
        # prefix for 'wikisource'.
        if site.code == 'sv':
            d = self.known_families.copy()
            d.pop('s')
            d['src'] = 'wikisource'
            return d
        else:
            return self.known_families

    def code2encodings(self, code):
        """Return a list of historical encodings for a specific site."""
        # Historic compatibility
        if code == 'pl':
            return 'utf-8', 'iso8859-2'
        if code == 'ru':
            return 'utf-8', 'iso8859-5'
        if code in self.latin1old:
            return 'utf-8', 'iso-8859-1'
        return self.code2encoding(code)



# -*- coding: utf-8 -*-
"""
This module contained backports to support older Python versions.

Their usage is deprecated and this module could be dropped soon.
"""
#
# (C) Pywikibot team, 2014-2018
#
# Distributed under the terms of the MIT license.
#

from __future__ import absolute_import, division, unicode_literals

from difflib import _format_range_unified
import logging

from pywikibot.tools import deprecated


@deprecated('difflib._format_range_unified', since='20160111')
def format_range_unified(start, stop):
    """
    Convert range to the "ed" format.

    DEPRECATED (Python 2.6 backport).
    Use difflib._format_range_unified instead.
    """
    return _format_range_unified(start, stop)


@deprecated('logging.NullHandler', since='20160111')
class NullHandler(logging.NullHandler):

    """This handler does nothing."""

    pass


@deprecated('logging.captureWarnings', since='20160111')
def captureWarnings(capture):
    """
    Capture warnings into logging.

    DEPRECATED (Python 2.6 backport).
    Use logging.captureWarnings instead.
    """
    logging.captureWarnings(capture)



# -*- coding: utf-8 -*-
"""Family module for LyricWiki."""
#
# (C) Pywikibot team, 2007-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot import family
from pywikibot.tools import classproperty, deprecated


# The LyricWiki family

# user-config.py:
# usernames['lyricwiki']['en'] = 'user'
class Family(family.SubdomainFamily, family.FandomFamily):

    """Family class for LyricWiki."""

    name = 'lyricwiki'
    domain = 'lyrics.fandom.com'
    codes = ('en', 'ru')

    @deprecated('APISite.version()', since='20141225')
    def version(self, code):
        return '1.19.24'

    def protocol(self, code):
        """Return 'https' as the protocol."""
        return 'https'

    @classproperty
    def langs(cls):
        """Property listing family languages."""
        return {code: cls.domain for code in cls.codes}



# -*- coding: utf-8 -*-
"""Installer script for Pywikibot 3.0 framework."""
#
# (C) Pywikibot team, 2009-2019
#
# Distributed under the terms of the MIT license.
#
from __future__ import (absolute_import, division,
                        print_function, unicode_literals)

import os
import sys

from setuptools import find_packages, setup

PYTHON_VERSION = sys.version_info[:3]
PY2 = (PYTHON_VERSION[0] == 2)

versions_required_message = """
Pywikibot is not available on:
{version}

This version of Pywikibot only supports Python 2.7.4+ or 3.4+.
"""


def python_is_supported():
    """Check that Python is supported."""
    # Any change to this must be copied to pwb.py
    return PYTHON_VERSION >= (3, 4, 0) or PY2 and PYTHON_VERSION >= (2, 7, 4)


if not python_is_supported():
    raise RuntimeError(versions_required_message.format(version=sys.version))

test_deps = ['bz2file', 'mock']

dependencies = ['requests>=2.20.0']

pydocstyle = 'pydocstyle<=3.0.0' if PY2 else 'pydocstyle>=2.5.0'
if PY2:
    pillow = 'Pillow<7.0.0'
elif PYTHON_VERSION < (3, 5):
    pillow = 'Pillow<6.0.0'
else:
    pillow = 'Pillow'

extra_deps = {
    # Core library dependencies
    'eventstreams': ['sseclient>=0.0.18,!=0.0.23'],
    'isbn': ['python-stdnum'],
    'Graphviz': ['pydot>=1.2'],
    'Google': ['google>=1.7'],
    'IRC': ['irc'],
    'mwparserfromhell': ['mwparserfromhell>=0.3.3'],
    'Tkinter': [pillow],
    'security': ['requests[security]', 'pycparser!=2.14'],
    'mwoauth': ['mwoauth>=0.2.4,!=0.3.1'],
    'html': ['BeautifulSoup4'],
    'flake8': [  # Due to incompatibilities between packages the order matters.
        'flake8>=3.7.5',
        pydocstyle,
        'hacking',
        'flake8-coding',
        'flake8-comprehensions',
        'flake8-docstrings>=1.1.0',
        'flake8-future-import',
        'flake8-mock>=0.3',
        'flake8-print>=2.0.1',
        'flake8-quotes>=2.0.1',
        'flake8-string-format',
        'flake8-tuple>=0.2.8',
        'flake8-no-u-prefixed-strings>=0.2',
        'pep8-naming>=0.7',
        'pyflakes>=2.1.0',
    ]
}

if PY2:
    # Additional core library dependencies which are only available on Python 2
    extra_deps.update({
        'csv': ['unicodecsv'],
    })

script_deps = {
    'flickrripper.py': ['flickrapi', pillow],
    'states_redirect.py': ['pycountry'],
    'weblinkchecker.py': ['memento_client>=0.5.1,!=0.6.0'],
    'patrol.py': ['mwparserfromhell>=0.3.3'],
}

# lunatic-python is only available for Linux
if sys.platform.startswith('linux'):
    script_deps['script_wui.py'] = ['irc', 'lunatic-python', 'crontab']

# The main pywin32 repository contains a Python 2 only setup.py with a small
# wrapper setup3.py for Python 3.
# http://pywin32.hg.sourceforge.net:8000/hgroot/pywin32/pywin32
# The main pywinauto repository doesn't support Python 3.
# The repositories used below have a Python 3 compliant setup.py
dependency_links = [
    'git+https://github.com/AlereDevices/lunatic-python.git#egg='
    'lunatic-python',
    'hg+https://bitbucket.org/TJG/pywin32#egg=pywin32',
    'git+https://github.com/vasily-v-ryabov/pywinauto-64#egg=pywinauto',
]

if PY2:
    # tools.ip does not have a hard dependency on an IP address module,
    # as it falls back to using regexes if one is not available.
    # The functional backport of py3 ipaddress is acceptable:
    # https://pypi.org/project/ipaddress
    # However the Debian package python-ipaddr is also supported:
    # https://pypi.org/project/ipaddr
    # Other backports are likely broken.
    # ipaddr 2.1.10+ is distributed with Debian and Fedora. See T105443.
    dependencies.append('ipaddr>=2.1.10')

    # version.package_version() uses pathlib which is a python 3 library.
    # pathlib2 is required for python 2.7
    dependencies.append('pathlib2')

    if (2, 7, 6) < PYTHON_VERSION < (2, 7, 9):
        # Python versions before 2.7.9 will cause urllib3 to trigger
        # InsecurePlatformWarning warnings for all HTTPS requests. By
        # installing with security extras, requests will automatically set
        # them up and the warnings will stop. See
        # <https://urllib3.readthedocs.org/en/latest/security.html#insecureplatformwarning>
        # for more details.
        # There is no secure version of cryptography for Python 2.7.6 or older.
        dependencies += extra_deps['security']

    script_deps['data_ingestion.py'] = extra_deps['csv']

try:
    import bz2
except ImportError:
    # Use bz2file if the python is not compiled with bz2 support.
    dependencies.append('bz2file')
else:
    _unused = bz2


# Some of the ui_tests depend on accessing the console window's menu
# to set the console font and copy and paste, achieved using pywinauto
# which depends on pywin32.
# These tests may be disabled because pywin32 depends on VC++, is time
# consuming to build, and the console window can't be accessed during appveyor
# builds.
# Microsoft makes available a compiler for Python 2.7
# http://www.microsoft.com/en-au/download/details.aspx?id=44266
if os.name == 'nt' and os.environ.get('PYSETUP_TEST_NO_UI', '0') != '1':
    # which isn't provided on pypi.
    test_deps += ['pywin32', 'pywinauto>=0.4.0']

extra_deps.update(script_deps)

# Add all dependencies as test dependencies,
# so all scripts can be compiled for script_tests, etc.
if 'PYSETUP_TEST_EXTRAS' in os.environ:
    test_deps += [i for k, v in extra_deps.items() if k != 'flake8' for i in v]

    if 'requests[security]' in test_deps:
        # Bug T105767 on Python 2.7 release 9+
        if PY2 and PYTHON_VERSION[2] >= 9:
            test_deps.remove('requests[security]')

# These extra dependencies are needed other unittest fails to load tests.
if PY2:
    test_deps += extra_deps['csv']
else:
    test_deps += ['six']


def get_version():
    """Get a valid pywikibot module version string."""
    version = '3.0'
    try:
        import subprocess
        date = subprocess.check_output(
            ['git', 'log', '-1', '--format=%ci']).strip()
        date = date.decode().split(' ')[0].replace('-', '')
        version += '.' + date
        if 'sdist' not in sys.argv:
            version += '.dev0'
    except Exception as e:
        print(e)
        version += '.dev0'
    return version


def read_desc(filename):
    """Read long description.

    Combine included restructured text files which must be done before
    uploading because the source isn't available after creating the package.
    """
    desc = []
    with open(filename) as f:
        for line in f:
            if line.strip().startswith('.. include::'):
                include = os.path.relpath(line.rsplit('::')[1].strip())
                if os.path.exists(include):
                    with open(include) as g:
                        desc.append(g.read())
                else:
                    print('Cannot include {0}; file not found'.format(include))
            else:
                desc.append(line)
    return ''.join(desc)


name = 'pywikibot'
setup(
    name=name,
    version=get_version(),
    description='Python MediaWiki Bot Framework',
    long_description=read_desc('README.rst'),
    keywords=('API', 'bot', 'framework', 'mediawiki', 'pwb', 'python',
              'pywikibot', 'pywikipedia', 'pywikipediabot', 'wiki',
              'wikimedia', 'wikipedia'),
    maintainer='The Pywikibot team',
    maintainer_email='pywikibot@lists.wikimedia.org',
    license='MIT License',
    packages=[str(name)] + [package
                            for package in find_packages()
                            if package.startswith('pywikibot.')],
    python_requires='>=2.7.4, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    install_requires=dependencies,
    dependency_links=dependency_links,
    extras_require=extra_deps,
    url='https://www.mediawiki.org/wiki/Manual:Pywikibot',
    download_url='https://tools.wmflabs.org/pywikibot/',
    test_suite='tests.collector',
    tests_require=test_deps,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Wiki',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    use_2to3=False
)



# -*- coding: utf-8 -*-
"""
WARNING: THIS MODULE EXISTS SOLELY TO PROVIDE BACKWARDS-COMPATIBILITY.

Do not use in new scripts; use the source to find the appropriate
function/method instead.

"""
#
# (C) Pywikibot team, 2008-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

from pywikibot.page import User
from pywikibot.tools import ModuleDeprecationWrapper

__all__ = ('User',)

wrapper = ModuleDeprecationWrapper(__name__)
wrapper._add_deprecated_attr('User',
                             replacement_name='pywikibot.User',
                             since='20141209')



# -*- coding: utf-8 -*-
"""
WARNING: THIS MODULE EXISTS SOLELY TO PROVIDE BACKWARDS-COMPATIBILITY.

Do not use in new scripts; use the source to find the appropriate
function/method instead.

"""
#
# (C) Pywikibot team, 2008-2018
#
# Distributed under the terms of the MIT license.
#
from __future__ import absolute_import, division, unicode_literals

import pywikibot
from pywikibot.data import api
from pywikibot.tools import deprecated, deprecate_arg

import io


@deprecated('pywikibot.data.api.Request', since='20120603')
@deprecate_arg('useAPI', None)
@deprecate_arg('retryCount', None)
@deprecate_arg('encodeTitle', None)
def GetData(request, site=None, back_response=False):
    """
    Query the server with the given request dict.

    DEPRECATED: Use pywikibot.data.api.Request instead.
    """
    if site:
        request['site'] = site

    req = api.Request(**request)
    result = req.submit()

    if back_response:
        pywikibot.warning('back_response is no longer supported; an empty '
                          'response object will be returned')
        res_dummy = io.StringIO()
        res_dummy.__dict__.update({'code': 0, 'msg': ''})
        return res_dummy, result
    return result
